#!/usr/bin/env perl

use v5.14;
use strict;
use utf8;
use warnings;
use Data::Dumper::Simple;
use constant TWOPI => 8 * atan2( 1, 1 );
use constant CX    => 75.0;
use constant CY    => 75.0;
my $id = 1;

my $radius = 100;		# intentionally huge
my @sym    = qw/4 5 6/;
my @mult   = qw/2 4 6 8 10 12/;
my %h;
foreach my $x (@sym) {
    foreach my $y (@mult) {
        $h{ $x * $y }++;
    }
}
push @sym, qw/7 9/;
$h{'28'}++;
$h{'36'}++;
$h{'42'}++;
my @lines = sort { $a <=> $b } ( map { $_ + 0 } keys(%h) );

# 4 : 8 12 16 20 24 28 32 36 40 48 60 72
# 5 : 10 20 30 40 50 60
# 6 : 12 24 30 36 42 48 60 72
# 7 : 28 42
# 9 : 36 72

foreach my $s (@sym) {    # for all symmetry groups
    foreach my $m ( grep { $_ % $s == 0 } @lines ) {  # for line counts in group
        my @othersym = grep { $m % $_ == 0 && $_ != $s } @sym;    # other syms?
        push @othersym, 0 if ( $#othersym < 0 );
        foreach my $osym (@othersym) {
            my $label = sprintf( "%d-%02d-%d", $s, $m, $osym );

            # label is for output svg file name
            say "\n", '# ', $label, ' :', "\n";
            open( my $fh, ">", $label . '.svg' )
              or die "cannot open > $label.svg: $!";

            # svg preamble & open group element
            say $fh '<?xml version="1.0" encoding="UTF-8" standalone="no"?>';
            say $fh
'<svg xmlns:svg="http://www.w3.org/2000/svg" xmlns:sodipodi="http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd" xmlns:inkscape="http://www.inkscape.org/namespaces/inkscape" height="150mm" version="1.0" id="',
              $id++, '" width="150mm">';
            say $fh qq{<defs id="defs4230">
    <clipPath id="clipPath4252">
    <rect height="100mm" id="arglebargle5" style="stroke: black; stroke-width: 0.0mm; fill: none;" width="100mm" x="25mm" y="25mm"/>
    </clipPath>
    <clipPath id="clipPath4253">
<circle cx="75mm" cy="75mm" id="arglebargle4" r="50mm" style="stroke: black; stroke-width: 0.0mm; fill: none;"/>
    </clipPath>
  </defs>
};

            say $fh join(
                "\n ", '<!-- ',
                $label . '.svg - geometric art template for InkScape',
                join( '',
                    $s,    '-fold symmetry, ',
                    $m,    ' total lines, ',
                    $osym, '-fold alternative symmetry' ),
                'Stewart C. Russell - scruss.com - 2015-08',
                'WTFPL (srsly)',
                ' -->'
            );

            # set up snapping, units, etc
            say $fh sprintf(
                qq{<sodipodi:namedview
     id="namedview%d"
     inkscape:object-nodes="true"
     inkscape:snap-intersection-paths="true"
     inkscape:object-paths="true"
     inkscape:snap-smooth-nodes="false"
     inkscape:snap-center="true" inkscape:document-units="mm"
     inkscape:current-layer="drawing" />}, $id++
            );

            say $fh
'  <g inkscape:groupmode="layer" inkscape:label="construction" id="layer',
              $id++, '" style="stroke-opacity: 0.75">';

            if ( $s == 4 ) {
                say $fh
                  sprintf( qq{<g clip-path="url(#clipPath4252)" id="gq%d">},
                    $id++ );
            }
            else {
                say $fh
                  sprintf( qq{<g clip-path="url(#clipPath4253)" id="gq%d">},
                    $id++ );
            }

            for ( my $l = 0 ; $l < ( $m / 2 ) ; $l++ ) {
                my $angle = TWOPI / 4 + $l * TWOPI / $m;

                # blue  lines every line
                my $colour = 'blue';

                # black on other/alt sym repeats
                $colour = 'black'
                  if ( ( $osym != 0 ) && ( ( $l * 2 ) % ( $m / $osym ) == 0 ) );

                # red lines on sym repeats
                $colour = 'red' if ( ( $l * 2 ) % ( $m / $s ) == 0 );

                # draw lines
                say $fh sprintf(
qq{    <line id="%d" x1="%.3fmm" y1="%.3fmm" x2="%.3fmm" y2="%.3fmm" style="stroke: %s; stroke-width: 0.1mm; fill: none;" />},
                    $id++,
                    CX - $radius * cos($angle),
                    CY - $radius * sin($angle),
                    CX + $radius * cos($angle),
                    CY + $radius * sin($angle),
                    $colour
                );
            }

            say $fh '</g>';    # close clipping path

            # draw square if base sym = 4
            if ( $s == 4 ) {
                say $fh '    <rect height="100mm" id="', $id++,
'" style="stroke: blue; stroke-width: 0.1mm; fill: none;" width="100mm" x="25mm" y="25mm"/>';
            }

            # draw circle
            say $fh '    <circle cx="75mm" cy="75mm" r="50mm" id="', $id++,
              '" style="stroke: blue; stroke-width: 0.1mm; fill: none;"/>';

            # close group element & svg postamble
            say $fh '  </g>';
            say $fh
'  <g inkscape:groupmode="layer" inkscape:label="drawing" id="layer',
              $id++, '" style="stroke-opacity: 1.0">';
            say $fh '    <!-- just to hold the active layer -->';
            say $fh '  </g>';
            say $fh '</svg>';
            close($fh);
        }
    }
}

exit;
